"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EudiWalletEventManager = void 0;
var _reactNative = require("react-native");
var _reactNativeUuid = _interopRequireDefault(require("react-native-uuid"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const {
  EudiWalletProxyEventsModule
} = _reactNative.NativeModules;
const {
  PROXY_EVENT_TYPE
} = EudiWalletProxyEventsModule.getConstants();
class EudiWalletEventManager {
  static nativeEventEmitter = new _reactNative.NativeEventEmitter(EudiWalletProxyEventsModule);
  static eventListeners = new Map();
  static start() {
    this.nativeEventEmitter.addListener(PROXY_EVENT_TYPE, proxyEvent => {
      this.eventListeners.forEach(listener => listener(proxyEvent));
    });
  }
  static stop() {
    var _this$nativeEventEmit;
    this.eventListeners.clear();
    (_this$nativeEventEmit = this.nativeEventEmitter) === null || _this$nativeEventEmit === void 0 || _this$nativeEventEmit.removeAllListeners(PROXY_EVENT_TYPE);
  }
  static addEventListener(eventListener) {
    const listenerId = _reactNativeUuid.default.v4();
    this.eventListeners.set(listenerId, eventListener);
    return listenerId;
  }
  static removeEventListener(listenerId) {
    if (!this.eventListeners.has(listenerId)) {
      throw new Error(`Cannot find event listener to remove (listenerId = ${listenerId}`);
    }
    this.eventListeners.delete(listenerId);
  }
}
exports.EudiWalletEventManager = EudiWalletEventManager;
//# sourceMappingURL=EudiWalletEventManager.js.map